% Program to implement Feedback Linearization for a robor manipulator.
% Create functions "controller" and "robosolver" before executing this program

clc;
clear all;
global K
global m1 m2 L1 L2 g

%A and B matrices of linear system
A=[0 0 1 0;0 0 0 1;0 0 0 0;0 0 0 0];
B=[0 0;0 0;1 0;0 1];
Q=[1 0 0 0;0 0.1 0 0;0 0 1 0;0 0 0 0.1];
R=[0.005 0;0 0.005];

[K] = lqr(A,B,Q,R);%linear controller parameters

% system parameters
m1=1; % first link end mass
m2=1; % second link end mass
L1=1; % length of first link
L2=1; % length of second link
g=9.8;% gravity

t0=0;
tf=8;
x0=[0.5 0.5 0 0]'; %initial state values
[t,theta]=ode45(@robosolver,[t0 tf],x0);

%desired signals
desired1 = sin(pi*t);
desired2 = cos(pi*t);

%error in the output
e1=desired1-theta(:,1);
e2=desired2-theta(:,2);

fprintf('\n\nMSE in the output of Link1: %.4f\n',mse(e1));
fprintf('MSE in the output of Link2: %.4f\n\n',mse(e2));

figure(1)
title('Desired,Actual Plots of theta1');
plot(t,desired1);
hold on;plot(t,theta(:,1),'r');
xlabel('Time'); ylabel('Theta1'); legend('Desired','Actual')


figure(2); 
title('Desired,Actual Plots of theta2');
plot(t,desired2);
hold on;plot(t,theta(:,2),'r');
xlabel('Time'); ylabel('Theta2'); legend('Desired','Actual')


figure(3);
title('Error variation over time');
plot(t,e1);
hold on;
plot(t,e2,'r');
xlabel('Time'); ylabel('Error'); legend('Err1','Err2');
