%Sliding mode control of robotic manipulator
%This program requires functions: "robo_solve" and "controller_slide"
clc;
clear all;

global lambda1 lambda2 k1 k2 
global m1 m2 L1 L2 g 

% switching matrix and controller parameters
lambda1=4;k1=4;k2=4;lambda2=3;
m1=1; % first &second link end mass
m2=1;
L1=1; % length of first link
L2=1;  % length of second link
g=9.8; 
t0=0;
tf=8;
x0=[1 0 0 0]';%initial values of angles and first derivatives

[t,theta]=ode45(@robo_solve,[t0 tf],x0);

ref1 = sin(pi*t);
ref2 = cos(pi*t);


figure(1)
plot(t,ref1);
hold on;plot(t,theta(:,1),'r');
xlabel('Time'); ylabel('Theta1'); legend('Desired','Actual')
%title('Desired,Actual Plots of theta1')

figure(2); 
plot(t,ref2);
hold on;plot(t,theta(:,2),'r');
xlabel('Time'); ylabel('Theta2'); legend('Desired','Actual')
%title('Desired,Actual Plots of theta2')

% Calculating and Plotting control inputs TAU1 and TAU2

ref1Dot = pi*cos(pi*t);
ref2Dot = -pi*sin(pi*t);
ref1dDot =-pi^2*sin(pi*t);
ref2dDot =-pi^2*cos(pi*t);
E1 = ref1-theta(:,1);
E2 = ref2-theta(:,2);
E3 = ref1Dot-theta(:,3);
E4 = ref2Dot-theta(:,4);

%conatants
alpha=(m1+m2)*L1^2;
beta=m2*L2*L2;
eta=m2*L1*L2;
e1=g/L1;

%Plant parameters
M11=alpha+beta+2*eta*cos(theta(:,2));
M12=beta+eta*cos(theta(:,2));
M21=M12;
M22=beta;

N1=-eta*(2*theta(:,3).*theta(:,4)+theta(:,4).*theta(:,4)).*sin(theta(:,2)) + alpha*e1*cos(theta(:,1))+eta*e1*cos(theta(:,1)+theta(:,2));
N2=eta*theta(:,3).*theta(:,3).*sin(theta(:,2))+eta*e1*cos(theta(:,1)+theta(:,2));

temp1=lambda1*E3+ref1dDot+k1*sign(E3+lambda1*E1);
temp2=lambda2*E4+ref2dDot+k2*sign(E4+lambda2*E2);

%control input
TAU1 =N1+(M11.*temp1+M12.*temp2);
TAU2 =N2+(M21.*temp1+M22.*temp2);

figure(3)
plot(t,TAU1);
xlabel('Time'); ylabel('TAU1');
title('Control input')

figure(4)
plot(t,TAU2);
xlabel('Time'); ylabel('TAU2'); 
title('Control input')
